@@ -1,13 +1,13 @@
-"""Non-standard filters."""
+"""Legacy implementations of non-standard filters."""
 
 try:
     from markupsafe import Markup
 except ImportError:
-    from liquid.exceptions import Markup
+    from liquid.exceptions import Markup  # type: ignore
 
 from .string import StringFilter
 
-# pylint: disable=arguments-differ too-few-public-methods
+# pylint: disable=too-few-public-methods arguments-differ
 
 
 class Safe(StringFilter):
