@@ -18,8 +18,17 @@ from typing import Tuple
 from typing import Generic
 from typing import TypeVar
 
+try:
+    from markupsafe import Markup
+except ImportError:
+    from liquid.exceptions import Markup  # type: ignore
+
 from liquid.context import Context
-from liquid.exceptions import LiquidTypeError, Error
+
+from liquid.exceptions import LiquidTypeError
+from liquid.exceptions import Error
+from liquid.exceptions import NoSuchFilterFunc
+from liquid.exceptions import FilterValueError
 
 # pylint: disable=missing-class-docstring too-few-public-methods
 
@@ -27,8 +36,6 @@ from liquid.exceptions import LiquidTypeError, Error
 class Expression(ABC):
     __slots__ = ()
 
-    expression_statement = True
-
     @abstractmethod
     def evaluate(self, context: Context) -> object:
         """Evaluate the expression with the given context."""
@@ -57,7 +64,7 @@ class Empty(Expression):
     __slots__ = ()
 
     def __eq__(self, other: object):
-        if isinstance(other, (Empty, Blank)):
+        if isinstance(other, Empty):
             return True
         if isinstance(other, (list, dict, str)) and not other:
             return True
@@ -75,30 +82,59 @@ class Empty(Expression):
 
 EMPTY = Empty()
 
-# As far as I'm able to tell, the reference implementation treats `empty`
-# and `blank` as an alias for the empty string. Are they simply converting
-# empty collections to strings (empty strings in the case of empty collections)
-# before comparing they to `blank` and/or `empty`?
 
-
-class Blank(Empty):
+class Blank(Expression):
     __slots__ = ()
 
+    def __eq__(self, other: object):
+        if isinstance(other, str) and (not other or other.isspace()):
+            return True
+        if isinstance(other, (list, dict)) and not other:
+            return True
+        if isinstance(other, Blank):
+            return True
+
+        return False
+
     def __repr__(self):  # pragma: no cover
         return "Blank()"
 
     def __str__(self):  # pragma: no cover
         return "blank"
 
+    def evaluate(self, context: Context) -> Blank:
+        return self
+
 
 BLANK = Blank()
 
 
+class Continue(Expression):
+    __slots__ = ()
+
+    def __eq__(self, other: object):
+        if isinstance(other, Continue):
+            return True
+        return False
+
+    def __repr__(self):  # pragma: no cover
+        return "Continue()"
+
+    def __str__(self):  # pragma: no cover
+        return "continue"
+
+    def evaluate(self, context: Context) -> int:
+        return 0
+
+
+CONTINUE = Continue()
+
+
 T = TypeVar("T")  # pylint: disable=invalid-name
 
 
 class Literal(Expression, Generic[T]):
-    __slots__ = "value"
+    __slots__ = ("value",)
 
     def __init__(self, value: T):
         self.value = value
@@ -142,6 +178,11 @@ class StringLiteral(Literal):
     def __repr__(self):  # pragma: no cover
         return f"StringLiteral(value='{self.value}')"
 
+    def evaluate(self, context: Context) -> Union[str, Markup]:
+        if context.autoescape:
+            return Markup(self.value)
+        return self.value
+
 
 class IntegerLiteral(Literal):
     __slots__ = ()
@@ -186,7 +227,7 @@ IdentifierPath = List[Union[IdentifierPathElement, "Identifier"]]
 
 
 class Identifier(Expression):
-    __slots__ = "path"
+    __slots__ = ("path",)
 
     def __init__(self, path: IdentifierPath):
         self.path = path
@@ -389,19 +430,26 @@ class FilteredExpression(Expression):
         result = self.expression.evaluate(context)
 
         for fltr in self.filters:
-            func = context.filter(fltr.name)
+            try:
+                func = context.filter(fltr.name)
+            except NoSuchFilterFunc:
+                if context.env.strict_filters:
+                    raise
+                continue
+
+            # Any exception causes us to abort the filter chain and discard the result.
+            # Nothing will be rendered.
             try:
                 args = fltr.evaluate_args(context)
                 kwargs = fltr.evaluate_kwargs(context)
                 result = func(result, *args, **kwargs)
-            except Exception as err:
-                # Any exception causes us to abort the filter chain and discard the
-                # result. Nothing will be rendered.
-                if not issubclass(type(err), Error):
-                    raise Error(
-                        f"filter '{fltr.name}': unexpected error: {err}"
-                    ) from err
+            except FilterValueError:
+                # Pass over filtered expressions who's left value is not allowed.
+                continue
+            except Error:
                 raise
+            except Exception as err:
+                raise Error(f"filter '{fltr.name}': unexpected error: {err}") from err
 
         return result
 
@@ -434,7 +482,7 @@ class AssignmentExpression(Expression):
         return ""
 
 
-LoopArgument = Optional[Union[IntegerLiteral, Identifier]]
+LoopArgument = Optional[Union[IntegerLiteral, Identifier, Continue]]
 
 
 class LoopExpression(Expression):
@@ -521,14 +569,29 @@ class LoopExpression(Expression):
         )
 
     def evaluate(self, context: Context) -> Tuple[Iterator[Any], int]:
+        # For the sake of the special for loop offset `continue`, we need to derive an
+        # identifier for this loop and store the theoretical stop index on the render
+        # context using that identifier.
+        #
+        # For compatibility with the reference implementation, we'll key loop stop
+        # indexes on the loop item name and the name of the iterable. Not the contents
+        # of the iterable. In the case of a range expression, we'll use a string
+        # representation of the expression.
+        _offset_key = [
+            self.name,
+        ]
+
         if self.identifier:
             # An identifier that must resolve to a list or dict in the current
             # global or local namespaces.
+            assert isinstance(self.identifier, Identifier)
+            _offset_key.append(str(self.identifier))
+
             obj = self.identifier.evaluate(context)
 
             if isinstance(obj, abc.Mapping):
                 length = len(obj)
-                loop_iter = obj.items()
+                loop_iter: Iterator[Any] = iter(obj.items())
             elif isinstance(obj, abc.Sequence):
                 length = len(obj)
                 loop_iter = iter(obj)
@@ -547,24 +610,36 @@ class LoopExpression(Expression):
             assert isinstance(start, int)
             assert isinstance(stop, int)
 
+            # Python's range function finishes on `stop - step`, whereas Liquid ranges
+            # are inclusive of stop, and always step by one, so add one.
             stop = stop + 1
 
-            loop_iter = range(start, stop)
+            _offset_key.append(f"{start}..{stop}")
+
+            loop_iter = iter(range(start, stop))
             length = stop - start
 
-        limit = self.limit
-        offset = self.offset
+        limit: Optional[int] = None
+        offset: Optional[int] = None
+        offset_key = "-".join(_offset_key)
 
-        if limit:
-            limit = limit.evaluate(context)
-            assert isinstance(limit, int)
-            length = min(length, limit)
+        if self.offset:
+            if self.offset == CONTINUE:
+                offset = context.stopindex(key=offset_key)
+            else:
+                _offset = self.offset.evaluate(context)
+                assert isinstance(_offset, int)
+                offset = _offset
 
-        if offset:
-            offset = offset.evaluate(context)
-            assert isinstance(offset, int)
             length = max(length - offset, 0)
 
+        if self.limit:
+            _limit = self.limit.evaluate(context)
+            assert isinstance(_limit, int)
+            limit = _limit
+            length = min(length, limit)
+
+        context.stopindex(key=offset_key, index=length)
         loop_iter = islice(loop_iter, offset, limit)
 
         if self.reversed:
@@ -589,18 +664,6 @@ def eval_number_expression(left: Number, operator: str, right: Number) -> bool:
     raise LiquidTypeError(f"unknown operator {left} {operator} {right}")
 
 
-def eval_empty_expression(left: Any, operator: str, right: Any) -> bool:
-    if not isinstance(left, Empty):
-        left, right = right, left
-
-    if operator == "==":
-        return left == right
-    if operator in ("!=", "<>"):
-        return left != right
-
-    raise LiquidTypeError(f"unknown operator {left} {operator} {right}")
-
-
 def is_truthy(obj: Any) -> bool:
     """Return True if the given object is Liquid truthy."""
     if obj in (False, None):
@@ -609,13 +672,16 @@ def is_truthy(obj: Any) -> bool:
 
 
 def compare(left: Any, operator: str, right: Any) -> bool:
-    if isinstance(left, Empty) or isinstance(right, Empty):
-        return eval_empty_expression(left, operator, right)
-
     if operator == "and":
         return is_truthy(left) and is_truthy(right)
     if operator == "or":
         return is_truthy(left) or is_truthy(right)
+
+    # NOTE: If both left and right are StrictUndefined, right will raise an exception
+    # before left.
+    if isinstance(right, (Empty, Blank)):
+        left, right = right, left
+
     if operator == "==":
         return left == right
     if operator in ("!=", "<>"):
