@@ -1,6 +1,7 @@
 """Test math filter functions."""
 import unittest
 
+from functools import partial
 from inspect import isclass
 
 from typing import NamedTuple
@@ -14,6 +15,18 @@ from liquid.exceptions import Error
 from liquid.exceptions import FilterArgumentError
 from liquid.exceptions import FilterValueError
 
+from liquid.builtin.filters._array import join
+from liquid.builtin.filters._array import first
+from liquid.builtin.filters._array import last
+from liquid.builtin.filters._array import concat
+from liquid.builtin.filters._array import map_
+from liquid.builtin.filters._array import reverse
+from liquid.builtin.filters._array import sort
+from liquid.builtin.filters._array import sort_natural
+from liquid.builtin.filters._array import where
+from liquid.builtin.filters._array import uniq
+from liquid.builtin.filters._array import compact
+
 from liquid.builtin.filters.array import Join
 from liquid.builtin.filters.array import First
 from liquid.builtin.filters.array import Last
@@ -43,7 +56,22 @@ class ArrayFilterTestCase(unittest.TestCase):
 
     def _test(self, filter_cls, test_cases):
         """Helper method for running lists of `Case`s"""
-        func = filter_cls(self.env)
+        with self.assertWarns(DeprecationWarning):
+            func = filter_cls(self.env)
+
+        for case in test_cases:
+            with self.subTest(msg=case.description):
+                if isclass(case.expect) and issubclass(case.expect, Error):
+                    with self.assertRaises(case.expect):
+                        func(case.val, *case.args, **case.kwargs)
+                else:
+                    self.assertEqual(
+                        func(case.val, *case.args, **case.kwargs), case.expect
+                    )
+
+    def _test_newstyle_filter(self, func, test_cases):
+        if getattr(func, "with_environment", False):
+            func = partial(func, environment=self.env)
 
         for case in test_cases:
             with self.subTest(msg=case.description):
@@ -138,6 +166,7 @@ class ArrayFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Join, test_cases)
+        self._test_newstyle_filter(join, test_cases)
 
     def test_first(self):
         """Test `first` filter function."""
@@ -188,6 +217,7 @@ class ArrayFilterTestCase(unittest.TestCase):
         ]
 
         self._test(First, test_cases)
+        self._test_newstyle_filter(first, test_cases)
 
     def test_last(self):
         """Test `last` filter function."""
@@ -238,6 +268,7 @@ class ArrayFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Last, test_cases)
+        self._test_newstyle_filter(last, test_cases)
 
     def test_concat(self):
         """Test `concat` filter function."""
@@ -302,6 +333,7 @@ class ArrayFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Concat, test_cases)
+        self._test_newstyle_filter(concat, test_cases)
 
     def test_map(self):
         """Test `map` filter function."""
@@ -366,6 +398,7 @@ class ArrayFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Map, test_cases)
+        self._test_newstyle_filter(map_, test_cases)
 
     def test_reverse(self):
         """Test `reverse` filter function."""
@@ -416,6 +449,7 @@ class ArrayFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Reverse, test_cases)
+        self._test_newstyle_filter(reverse, test_cases)
 
     def test_sort(self):
         """Test `sort` filter function."""
@@ -480,6 +514,7 @@ class ArrayFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Sort, test_cases)
+        self._test_newstyle_filter(sort, test_cases)
 
     def test_sort_natural(self):
         """Test `sort_natural` filter function."""
@@ -551,6 +586,7 @@ class ArrayFilterTestCase(unittest.TestCase):
         ]
 
         self._test(SortNatural, test_cases)
+        self._test_newstyle_filter(sort_natural, test_cases)
 
     def test_where(self):
         """Test `where` filter function."""
@@ -615,6 +651,7 @@ class ArrayFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Where, test_cases)
+        self._test_newstyle_filter(where, test_cases)
 
     def test_uniq(self):
         """Test `uniq` filter function."""
@@ -672,6 +709,7 @@ class ArrayFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Uniq, test_cases)
+        self._test_newstyle_filter(uniq, test_cases)
 
     def test_compact(self):
         """Test `compact` filter function."""
@@ -715,3 +753,4 @@ class ArrayFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Compact, test_cases)
+        self._test_newstyle_filter(compact, test_cases)
