@@ -1,5 +1,6 @@
 import unittest
 
+from functools import partial
 from inspect import isclass
 
 from typing import NamedTuple
@@ -10,6 +11,32 @@ from typing import Dict
 from liquid.environment import Environment
 from liquid.exceptions import FilterArgumentError
 
+# New style filters
+from liquid.builtin.filters._string import capitalize
+from liquid.builtin.filters._string import append
+from liquid.builtin.filters._string import downcase
+from liquid.builtin.filters._string import escape
+from liquid.builtin.filters._string import escape_once
+from liquid.builtin.filters._string import lstrip
+from liquid.builtin.filters._string import newline_to_br
+from liquid.builtin.filters._string import prepend
+from liquid.builtin.filters._string import remove
+from liquid.builtin.filters._string import remove_first
+from liquid.builtin.filters._string import replace
+from liquid.builtin.filters._string import replace_first
+from liquid.builtin.filters._string import slice_
+from liquid.builtin.filters._string import split
+from liquid.builtin.filters._string import upcase
+from liquid.builtin.filters._string import strip
+from liquid.builtin.filters._string import rstrip
+from liquid.builtin.filters._string import strip_html
+from liquid.builtin.filters._string import strip_newlines
+from liquid.builtin.filters._string import truncate
+from liquid.builtin.filters._string import truncatewords
+from liquid.builtin.filters._string import url_encode
+from liquid.builtin.filters._string import url_decode
+
+# Depreciated class-based filters
 from liquid.builtin.filters.string import Capitalize
 from liquid.builtin.filters.string import Append
 from liquid.builtin.filters.string import Downcase
@@ -51,7 +78,24 @@ class StringFilterTestCase(unittest.TestCase):
 
     def _test(self, filter_cls, test_cases):
         """Helper method for running lists of `Case`s"""
-        func = filter_cls(self.env)
+        with self.assertWarns(DeprecationWarning):
+            func = filter_cls(self.env)
+
+        for case in test_cases:
+            with self.subTest(msg=case.description):
+                if isclass(case.expect) and issubclass(
+                    case.expect, FilterArgumentError
+                ):
+                    with self.assertRaises(case.expect):
+                        func(case.val, *case.args, **case.kwargs)
+                else:
+                    self.assertEqual(
+                        func(case.val, *case.args, **case.kwargs), case.expect
+                    )
+
+    def _test_newstyle_filter(self, func, test_cases):
+        if getattr(func, "with_environment", False):
+            func = partial(func, environment=self.env)
 
         for case in test_cases:
             with self.subTest(msg=case.description):
@@ -100,6 +144,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Capitalize, test_cases)
+        self._test_newstyle_filter(capitalize, test_cases)
 
     def test_append(self):
         """Test append filter function."""
@@ -157,6 +202,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Append, test_cases)
+        self._test_newstyle_filter(append, test_cases)
 
     def test_downcase(self):
         """Test downcase filter function."""
@@ -193,6 +239,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Downcase, test_cases)
+        self._test_newstyle_filter(downcase, test_cases)
 
     def test_escape(self):
         """Test escape filter function."""
@@ -229,6 +276,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Escape, test_cases)
+        self._test_newstyle_filter(escape, test_cases)
 
     def test_escape_once(self):
         """Test escape_once filter function."""
@@ -272,6 +320,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(EscapeOnce, test_cases)
+        self._test_newstyle_filter(escape_once, test_cases)
 
     def test_lstrip(self):
         """Test lstrip filter function."""
@@ -322,6 +371,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(LStrip, test_cases)
+        self._test_newstyle_filter(lstrip, test_cases)
 
     def test_newline_to_br(self):
         """Test newline_to_br filter function."""
@@ -372,6 +422,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(NewlineToBR, test_cases)
+        self._test_newstyle_filter(newline_to_br, test_cases)
 
     def test_prepend(self):
         """Test prepend filter function."""
@@ -429,6 +480,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Prepend, test_cases)
+        self._test_newstyle_filter(prepend, test_cases)
 
     def test_remove(self):
         """Test remove filter function."""
@@ -486,6 +538,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Remove, test_cases)
+        self._test_newstyle_filter(remove, test_cases)
 
     def test_remove_first(self):
         """Test remove_first filter function."""
@@ -543,6 +596,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(RemoveFirst, test_cases)
+        self._test_newstyle_filter(remove_first, test_cases)
 
     def test_replace(self):
         """Test replace filter function."""
@@ -614,6 +668,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Replace, test_cases)
+        self._test_newstyle_filter(replace, test_cases)
 
     def test_replace_first(self):
         """Test replace_first filter function."""
@@ -685,6 +740,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(ReplaceFirst, test_cases)
+        self._test_newstyle_filter(replace_first, test_cases)
 
     def test_slice(self):
         """Test slice filter function."""
@@ -784,6 +840,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Slice, test_cases)
+        self._test_newstyle_filter(slice_, test_cases)
 
     def test_split(self):
         """Test split filter function."""
@@ -866,6 +923,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Split, test_cases)
+        self._test_newstyle_filter(split, test_cases)
 
     def test_upcase(self):
         """Test upcase filter function."""
@@ -902,6 +960,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Upcase, test_cases)
+        self._test_newstyle_filter(upcase, test_cases)
 
     def test_strip(self):
         """Test strip filter function."""
@@ -952,6 +1011,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Strip, test_cases)
+        self._test_newstyle_filter(strip, test_cases)
 
     def test_rstrip(self):
         """Test rstrip filter function."""
@@ -1002,6 +1062,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(RStrip, test_cases)
+        self._test_newstyle_filter(rstrip, test_cases)
 
     def test_strip_html(self):
         """Test strip_html filter function."""
@@ -1048,6 +1109,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(StripHTML, test_cases)
+        self._test_newstyle_filter(strip_html, test_cases)
 
     def test_strip_newlines(self):
         """Test strip_newlines filter function."""
@@ -1098,6 +1160,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(StripNewlines, test_cases)
+        self._test_newstyle_filter(strip_newlines, test_cases)
 
     def test_truncate(self):
         """Test truncate filter function."""
@@ -1169,6 +1232,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(Truncate, test_cases)
+        self._test_newstyle_filter(truncate, test_cases)
 
     def test_truncatewords(self):
         """Test truncatewords filter function."""
@@ -1282,6 +1346,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(TruncateWords, test_cases)
+        self._test_newstyle_filter(truncatewords, test_cases)
 
     def test_url_encode_html(self):
         """Test url_encode filter function."""
@@ -1318,6 +1383,7 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(URLEncode, test_cases)
+        self._test_newstyle_filter(url_encode, test_cases)
 
     def test_url_decode_html(self):
         """Test url_decode filter function."""
@@ -1354,3 +1420,4 @@ class StringFilterTestCase(unittest.TestCase):
         ]
 
         self._test(URLDecode, test_cases)
+        self._test_newstyle_filter(url_decode, test_cases)
